#  compute the eLC MLE for discrete Gaussian example in manuscript

#--------------------------
#  load required functions (assuming files in working directory)
#--------------------------

dyn.load("convhullnew.so")	 
source("functions.R")


#--------------------------
# generate discrete Gaussian sample, result written to emp.txt
#--------------------------
set.seed(20-11-1951)
xx 		<- 	rgaussiandscr(n=500)  # create discrete gaussian random sample
# optional input parameters:
# n: sample size
# N: (-N,N)^2 is the range for calculating the true PMF, by default N=100
# mu: mean of bivariate discrete gaussian, by default mu= c(1,2)      
# sig1: standard deviation of bivariate discrete gaussian, by default sig1 = 2
# sig2: standard deviation of bivariate discrete gaussian, by default sig2 = 5
# rho: correlation coefficient of bivariate discrete gaussian, by default rho = 0.6
xd		<-	as.matrix(aggregate(list(emp=rep(1,nrow(data.frame(xx)))), data.frame(xx), length))
write.table(xd, "emp.txt", sep = ",", row.names = F, col.names = T)   


#--------------------------
# estimate eLC MLE, result written to mle.txt
#--------------------------

xd 	<- as.matrix(read.table("emp.txt", header = TRUE, sep=","))
mle <- eLC(xd)   # compute eLC MLE  
# xd: input data, first two columns give the unique observed location, last column is the empirical count
colnames(mle) <- c("X","Y","elc")
write.table(mle, "mle.txt",sep = ",", row.names = F,col.names = T)

