#install.packages("logcondiscr")
#install.packages("mvtnorm")
library(logcondiscr)
library(mvtnorm)

rgaussiandscr <- function(n, N=100, mu=c(1,2), sig1=2, sig2=5, rho=0.6) {

  sig 	<- 	matrix(c(sig1^2, rho*sig1*sig2, rho*sig1*sig2, sig2^2), ncol=2)   
  x 		<- 	expand.grid(X=(mu[1]-N):(mu[1]+N), Y=(mu[2]-N):(mu[2]+N))
  p 		<- 	dmvnorm(x, mean=mu, sigma=sig)
  p		<-	p/sum(p)
  
  idx 	<- 	sample(1:nrow(x), size=n, replace=TRUE, prob=p)
  rs 	<- 	x[idx,]
  
  return(rs)
}

pgaussiandscr <- function(x, mu=c(1,2), sig1=2, sig2=5, rho=0.6) {

  sig 	<- 	matrix(c(sig1^2, rho*sig1*sig2, rho*sig1*sig2, sig2^2), ncol=2)   
  p 		<- 	dmvnorm(x, mean=mu, sigma=sig)
  p		<-	p/sum(p)
  
  return(p)
}

pNgaussiandscr <- function(N=100, mu=c(1,2), sig1=2, sig2=5, rho=0.6) {

  x 		<- 	expand.grid(X=(mu[1]-N):(mu[1]+N), Y=(mu[2]-N):(mu[2]+N))
  sig 	<- 	matrix(c(sig1^2, rho*sig1*sig2, rho*sig1*sig2, sig2^2), ncol=2)   
  p 		<- 	dmvnorm(x, mean=mu, sigma=sig)
  p		<-	p/sum(p)
  
  d <- cbind(x, p)
  colnames(d) <- c("X", "Y","true")
  return(d)
}


#################################################################
##Note this has identical functionality to the function "convhulln" in the package "geometry"
"convhullnew" <-
  function (p, options = "Qt" )
    .Call("convhullnew", as.matrix(p), as.character(options), PACKAGE="convhullnew")


## Initialize f
'initialf' <- 
  function(x,n,Xin,h){
    mean(apply(dnorm(rep(1,n)%o%x-Xin,sd = rep(1,n)%o%h),1,prod))
  }


## Find initial y
'initialy' <-
  function(Xin){
    n <- nrow(Xin)
    d <- ncol(Xin)
    h <- apply(Xin,2,sd)*n^(-1/(d+4))
    log(apply(Xin,1,initialf,n=n,Xin=Xin,h=h))
  }


########################################################################
# estimate eLC MLE
# xd: input data, first two columns are x, y; 3rd column is weights (empirical PMF).
########################################################################
eLC <- function(xd) {
  w <- xd[,3]/sum(xd[,3])     # weights, or empirical PMF
  x <- xd[,c(1,2)]
  
  #initial y
  for (j in 1:dim(x)[1])
    for (k in 1:dim(x)[2])
      x[j,k] <- as.numeric(x[j,k])
  
  y =initialy(x)
  
  # generate covexhull and verify out points
  chull <- convhullnew(x)
  outerpoints <- unique(c(chull))  # index of outerpoints
  
  innerpoints <- (1:nrow(x))[-outerpoints]  # inner points index of convex hull
  nouter <- length(outerpoints)   # number of outerpoints
  
  lcdsort <- c(outerpoints,innerpoints)   #new order
  
  # initial paramter under R enviornment for iteration   
  verbose=-1               # display control
  alpha=5                  # dilation factor
  c=1                      # for minimization; c for initial step length
  sigmatol=10^-8           # sigma function tolerance
  integraltol=10^-4        # integral tolerance
  ytol=10^-4               # MLE distance tolerance
  Jtol=0.001
  chtol = 10^-6 
  
  # transfer the paramters to formate under C environment
  opts <- rep(0,11)
  opts[1] <- as.double(-c)       # for minimization; c for initial step length
  opts[2] <- ytol                # MLE distance tolerance
  opts[3] <- sigmatol            # sigma function tolerance
  opts[4] <- 15000               # maximum number of iterations
  opts[5] <- as.double(verbose)  # display control
  opts[6] <- integraltol         # integral tolerance
  opts[7] <- as.double(alpha)    # dilation factor
  opts[8] <- 1*10^(-11)          # for numerical gradient approx
  
  chopts <- paste( "Qt", sep="" )  # Set up the qhull options
  
  # call c functions to estimate probablity of observations
  out <- .C( "logconestw",
             yvalue = as.double( y [ lcdsort ] ),
             as.double( x [ lcdsort, ] ),
             as.integer( ncol( x ) ),
             as.integer( nrow( x ) ),
             as.double( w [ lcdsort ] ),
             options = as.double( opts ),
             minvalue = double( 1 ),
             Jtol = as.double( Jtol ),
             chopts = as.character( chopts ),
             nouter = as.integer( nouter )
             ,PACKAGE="convhullnew" )
  
  # calculate mass for all discrete points inside convexhull.
  # output: cordinators of discrete points, and mass for each points.
  yall <- .Call("exph", as.double(out$yvalue), as.integer( ncol( x ) ),PACKAGE="convhullnew")
  return(yall)
}
